import sys
import unittest
sys.path.append('/home/bilal/AiCourse/Webscraping_Project/scraper')
from OQMD_new_version import CompoundScraper

class CompoundScraperTestCase(unittest.TestCase):
    def test_get_to_URL(self):
        root = "http://oqmd.org/api/search#apisearchresult"
        features = {'Name':[], 'Spacegroup':[], 'Volume':[], 'Band_gap':[]}
        scraper = CompoundScraper(n=1, root=root, list=[], features=features)
        scraper.get_to_URL()
    
    def test_load_data(self):
        # testing the type of the load_data output
        # compare the lengths of expected output list and actual list 
        root = "http://oqmd.org/api/search#apisearchresult"
        features = {'Name':[], 'Spacegroup':[], 'Volume':[], 'Band_gap':[]}
        scraper = CompoundScraper(n=1, root=root, list=[], features=features)
        scraper.get_to_URL()
        scraper.load_data()
        expected_list_type = type([]) # FIX IT
        actual_list_type = type(scraper.list)
        expected_list_len = 50
        actual_list_len = len(scraper.list)
        self.assertEqual(expected_list_type, actual_list_type)
        self.assertEqual(expected_list_len, actual_list_len)
    
    def test_extract_data(self):
        # check if the number of elements are the same
        # check if the elements are different from each other , use self.assertItemsEqual
        root = "http://oqmd.org/api/search#apisearchresult"
        features = {'Name':[], 'Spacegroup':[], 'Volume':[], 'Band_gap':[]}
        scraper = CompoundScraper(n=1, root=root, list=[], features=features)
        scraper.extract_data()
        actual_output = scraper.features
        expected_output = {'Name': ['CsHoSiS4',  'Lu',  'Tm',  'Ne',  'La',  'Pr',  'Kr',  'V',  'Rb',  'Ta',  'V',  'Ne',  'Al',  'Ne',  'K',  'Rh',  'Er',  'Zr',  'Ho',  'Lu',  'Tb',  'Ne',  'P',  'Ru',  'Ir',  'Gd',  'Fe',  'Ni',  'Fe',  'Tc',  'Re',  'Pt',  'YbSe',  'SiP',  'SmZn',  'SiC',  'U',  'Zr',  'Os',  'ZnNi',  'TiAg',  'ZnNi',  'YbTe',  'TmAg',  'UTe',  'YAg',  'LiBi',  'TcB',  'LiTl',  'YbAg'], 'Spacegroup': ['P212121',  'R-3m',  'R-3m',  'Fm-3m',  'Im-3m',  'Im-3m',  'Fm-3m',  'Fm-3m',  'Im-3m',  'Fm-3m',  'Im-3m',  'Fm-3m',  'Fm-3m',  'Fm-3m',  'Im-3m',  'Fm-3m',  'Im-3m',  'Im-3m',  'Im-3m',  'Im-3m',  'Im-3m',  'Fm-3m',  'Pm-3m',  'Fm-3m',  'Fm-3m',  'Im-3m',  'Im-3m',  'Im-3m',  'Fm-3m',  'Fm-3m',  'Fm-3m',  'Fm-3m',  'Fm-3m',  'F-43m',  'Pm-3m',  'Fm-3m',  'Im-3m',  'Fm-3m',  'Fm-3m',  'P4/mmm',  'P4/mmm',  'Pm-3m',  'Fm-3m',  'Pm-3m',  'Pm-3m',  'Pm-3m',  'P4/mmm',  'Pm-3m',  'Pm-3m',  'Pm-3m'], 'Volume': ['760.627',  '86.351',  '88.611',  '21.720',  '37.786',  '35.887',  '42.831',  '13.515',  '89.599',  '18.788',  '13.169',  '22.212',  '16.483',  '11.262',  '71.876',  '13.986',  '30.711',  '22.806',  '31.050',  '29.429',  '32.046',  '10.226',  '14.572',  '13.798',  '14.465',  '32.540',  '11.174',  '10.860',  '10.198',  '14.563',  '14.946',  '15.540',  '51.335',  '37.415',  '47.871',  '16.624',  '20.053',  '23.185',  '14.340',  '23.962',  '33.974',  '24.026',  '63.967',  '45.446',  '52.510',  '47.706',  '47.134',  '20.315',  '41.175',  '48.189'], 'Band_gap': ['3.024',  '0',  '0',  '11.910',  '0',  '0',  '7.487',  '0',  '0',  '0',  '0',  '11.657',  '0',  '13.500',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '14.115',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '2.083',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '0',  '1.620',  '0',  '0',  '0',  '0',  '0',  '0',  '0']}
        self.assertCountEqual(actual_output, expected_output)

unittest.main(argv=[''], verbosity=2, exit=False)






from sqlalchemy import create_engine
import pandas as pd
import time


def convert_to_DF(data_name, file_out : str, to_csv : bool):
    '''
    

    '''
    compounds_data = pd.DataFrame(data_name)
    with pd.option_context('display.max_rows', None, 'display.max_columns', None): 
        if to_csv == True:
            compounds_data.to_csv(file_out + '.csv')
        else:
            pass

        return compounds_data
    
def import_to_SQL(name='compounds_dataset'):
    '''
    Imports the output as an SQL database

    '''
    DATABASE_TYPE = 'postgresql'
    DBAPI = 'psycopg2'
    HOST = 'localhost'
    USER = 'postgres'
    PASSWORD = 'kenya123'
    DATABASE = 'Compounds_databse'
    PORT = 5432
    engine = create_engine(f"{DATABASE_TYPE}+{DBAPI}://{USER}:{PASSWORD}@{HOST}:{PORT}/{DATABASE}")

    compounds_data.to_sql(name, engine, if_exists='replace')


from numpy import add
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from sqlalchemy import create_engine
import pandas as pd
import time
from scraper.generic_scraper import Scraper

class CompoundScraper(Scraper):
    '''
    This class is used to represent a compound scraper.

    Features
    --------

    (1) Name 
    (2) Space Group
    (3) Volume
    (4) Band Gap


    Parameters
    ----------
    n       : int
              Defines the number of pages to extract data from
    
    root    : str
              Defines the URL to extract data from
    
    list    : list
              This initiates an empty list to append all necessary links
    
    features: dict
              A dictionary that defines the output for extracted target features

    Attributes
    ----------
    (1) n
    (2) root
    (3) list
    (4) features

    '''

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        

    def get_to_URL(self) -> None:
        
        '''
        This function opens the target URL prior to extraction.
        It depends on the parameter 'root'.

        '''

        self.driver.get(self.root)
        search_button = self.driver.find_element_by_xpath('//*[@id="submit-id-search"]')
        search_button.click()
        
    def load_data(self) -> None:
        '''
        This function loads all 
        the WebElements pending extraction.

        '''
        table_xpath = '//*[@id="ResultTable"]/tbody'
        table = WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.XPATH, table_xpath)))
        #table = self.driver.find_element_by_xpath('//*[@id="ResultTable"]/tbody')
        compound_list = table.find_elements_by_xpath('.//tr')[1:]
        for i in compound_list:
            self.list.append(i)

    def next_page_click(self):
        next_page_button = self.driver.find_element_by_xpath('//button[@class="next"]') 
        next_page_button.click()  


    def extract_data(self) -> None:
        '''
        This function extracts the target compound data 
        and stores it in a dictionary

        '''
        print('Extracting compounds data ...')
        
        print('Initialising ...')
        self.get_to_URL()

        for j in range(self.n):
            
            print(f"Extracting from Page_{j}")
            
            
            print('Loading data ...')
            # reload data due to stale element
            self.load_data()   

            for item in self.list:
                name = item.find_element_by_xpath('.//td[@width="60"]')
                spacegroup = item.find_elements_by_xpath('.//td[@width="50"]')[3]
                volume = item.find_elements_by_xpath('.//td[@width="80"]')[1]
                band_gap = item.find_elements_by_xpath('.//td[@width="50"]')[5]
                self.features['Name'].append(name.text)
                self.features['Spacegroup'].append(spacegroup.text)
                self.features['Volume'].append(volume.text)
                self.features['Band_gap'].append(band_gap.text)
                time.sleep(1)
            
            print(f"Going to page_{j} ...")
            self.next_page_click()
            time.sleep(10)
        
        print('Extraction Complete ... ')


        







from numpy import add
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from scraper.data_structure import convert_to_DF 
import time
from scraper.generic_scraper import Scraper

class PeriodicTableScraper(Scraper):
    
    '''
    This class is used to represent a periodic table scraper.
    
    Features
    --------

    (1) Name 
    (2) Space Group
    (3) Volume
    (4) Band Gap


    Parameters
    ---------
    n       : int
              Defines the number of periodic table elements to extract data from 
    
    root    : str
              Defines the URL to extract data from
    
    list    : list
              This initiates an empty list to append all necessary links
    
    features: dict
              A dictionary that defines the output for extracted target features

    Attributes
    ----------
    (1) n
    (2) root
    (3) list
    (4) features
    
    '''
    
    def __init__(self, **kwargs):
         super().__init__(**kwargs)

    def extract_links(self) -> None :
        
        '''
        Extracts all urls for each periodic table element 
        and stores them in a list.

        '''
        print(f" --- Extracting elemental URLs ---")
        
        self.driver.get(self.root)
        element_list = self.driver.find_elements_by_xpath("//li[@class='p-md-bottom print-avoid-break-inside print-padding-top']")
        for item in element_list[0:self.n]:
                link = item.find_element_by_xpath('.//a').get_attribute('href')
                self.list.append(link)
        
        print(f"... Done")


    def extract_data(self, to_DF=False) -> None :
        
        '''
        Extracts information on specified features of 
        elements from the periodic table.

        Parameters
        ----------
        to_DF : bool
                Serves as a switch for converting the output extract data to a Data Frame.

        '''
        
        self.extract_links()
        for i in self.list:
            self.driver.get(i)
            time.sleep(3)
            atomic_no_xpath = '//div[@class="f-15"]'
            element_name_xpath = '//*[@id="Element-Name"]/div[2]/div[1]/p'
            electronegativity_xpath = '//*[@id="Electronegativity"]/div[2]/div[1]/p'
            boiling_point_xpath = '//*[@id="Boiling-Point"]/div[2]/div[1]/p'

            try:
                atomic_n = WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((By.XPATH, atomic_no_xpath)))
                element_name = WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((By.XPATH, element_name_xpath)))
                electronegativity = WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((By.XPATH, electronegativity_xpath)))
                boiling_point = WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((By.XPATH, boiling_point_xpath)))
                self.features['Element_Name'].append(element_name.text)
                self.features['Atomic_Number'].append(atomic_n.text)
                self.features['Boiling_Point'].append(boiling_point.text)
                self.features['Electronegativity'].append(electronegativity.text)
            except TimeoutException: 
                self.features['Electronegativity'].append('NA')
                self.features['Boiling_Point'].append('NA')
        print(f"--- Extracting elemental features ---")
        print(f" ... Done")
        
        if to_DF == True:
            print(f"converting to DF ...")
            convert_to_DF(self.features, 'elements_data', to_csv=True)
        else:
            pass














#%%
from numpy import add
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.ui import WebDriverWait


class Scraper:

    '''
    This class represents a scraper.  

    Features
    --------

    (1) Name 
    (2) Space Group
    (3) Volume
    (4) Band Gap


    Parameters
    ---------
    n       : int
              Defines the number of pages to extract data from
    
    root    : str
              Defines the URL to extract data from
    
    list    : list
              This initiates an empty list to append all necessary links
    
    features: dict
              A dictionary that defines the output for extracted target features
              Example: features = {'Name':[], 'Spacegroup':[], 'Volume':[], 'Band_gap':[]}

    Attributes
    ----------
    (1) n
    (2) root
    (3) list
    (4) features
    
    '''
    
    def __init__(self, **kwargs):
        self.n = kwargs['n']
        self.root = kwargs['root']
        self.list = kwargs['list']
        self.features = kwargs['features']
        self.driver = webdriver.Chrome('chromedriver')
    




import unittest
import sys
sys.path.append('/home/bilal/AiCourse/Webscraping_Project/scraper')
from periodic_table_new import PeriodicTableScraper

class PeriodicTableScraperTestCase(unittest.TestCase):
    def test_extract_links(self) -> None:
        root = "https://pubchem.ncbi.nlm.nih.gov/periodic-table/#view=list"
        features = {'Element_Name':[], 'Atomic_Number':[], 'Electronegativity':[], 'Boiling_Point':[]}
        element_scraper = PeriodicTableScraper(n=5, root=root, list=[], features=features)
        element_scraper.extract_links()
        expected_output = ['https://pubchem.ncbi.nlm.nih.gov/element/1', 'https://pubchem.ncbi.nlm.nih.gov/element/2', 'https://pubchem.ncbi.nlm.nih.gov/element/3', 'https://pubchem.ncbi.nlm.nih.gov/element/4', 'https://pubchem.ncbi.nlm.nih.gov/element/5']
        actual_output = element_scraper.list
        self.assertCountEqual(expected_output, actual_output)
    
    def test_extract_data(self) -> None:
        root = "https://pubchem.ncbi.nlm.nih.gov/periodic-table/#view=list"
        features = {'Element_Name':[], 'Atomic_Number':[], 'Electronegativity':[], 'Boiling_Point':[]}
        element_scraper = PeriodicTableScraper(n=5, root=root, list=[], features=features)
        element_scraper.extract_data()
        actual_output = element_scraper.features
        expected_output = {'Element_Name': ['Hydrogen', 'Helium', 'Lithium', 'Beryllium', 'Boron'], 
                           'Atomic_Number': ['1', '2', '3', '4', '5'], 
                           'Electronegativity': ['2.2 (Pauling Scale)', '4.16 (Allen Scale)', '0.98 (Pauling Scale)', '1.57 (Pauling Scale)', '2.04 (Pauling Scale)'],
                           'Boiling_Point': ['20.28 K (-252.87°C or -423.17°F)', '4.22 K (-268.93°C or -452.07°F)', '1615 K (1342°C or 2448°F)',  '2744 K (2471°C or 4480°F)', '4273 K (4000°C or 7232°F)']}
        self.assertCountEqual(expected_output, actual_output)

unittest.main(argv=[''], verbosity=2, exit=False)








# %%



#%%
from scraper.OQMD_new_version import CompoundScraper
from scraper.periodic_table_new import PeriodicTableScraper
import argparse

'''

This is a script to execute a specific scraper instance based
on a selected url. For this scraper, there are only two choices of url;

(1) OQMD website - "http://oqmd.org/api/search#apisearchresult"
(2) Periodic table webiste -"https://pubchem.ncbi.nlm.nih.gov/periodic-table/#view=list"

This code can be executed by making use of the flags -r and '--root'
Example: python example.py -r [insert url]


'''

# parser = argparse.ArgumentParser()
# parser.add_argument("-r", "--root", dest="root", help="webpage url", type=str)
# args = parser.parse_args()


select_options = input('Select URL:  (1) Periodic table website  (2) OQMD website')

if select_options == str(2):
    root = "http://oqmd.org/api/search#apisearchresult"
    features = {'Name':[], 'Spacegroup':[], 'Volume':[], 'Band_gap':[]}
    scraper = CompoundScraper(n=2, root=root, list=[], features=features)
    scraper.extract_data()    

elif select_options == str(1): 
    root = "https://pubchem.ncbi.nlm.nih.gov/periodic-table/#view=list"
    features = {'Element_Name':[], 'Atomic_Number':[], 'Electronegativity':[], 'Boiling_Point':[]}
    scraper = PeriodicTableScraper(n=5, root=root, list=[], features=features)
    scraper.extract_data(to_DF=True)
else:
    print(f"Enter a valid URL")
    


# %%
scraper.features
# %%



from setuptools import setup
from setuptools import find_packages

setup(
    name='el-compX-scraper', ## This will be the name your package will be published with
    version='0.1.3', 
    description='This package scrapes features of elements and compounds from specified websites',
    url='https://github.com/barmiy01/Webscraping_Project.git', # Add the URL of your github repo if published 
                                                                   # in GitHub
    author='Bilal Armiyawo', # Your name
    license='MIT',
    packages=find_packages(), # This one is important to explain. See the notebook for a detailed explanation
    install_requires=['requests', 'beautifulsoup4', 'selenium', 'pandas', 'sqlalchemy'], # For this project we are using two external libraries
                                                     # Make sure to include all external libraries in this argument
)